from pydantic import BaseModel

from ..types.connection import TileConnectionType
from ..types.tile import TileType


class Tile(BaseModel):
    id: str
    tile_type: TileType
    rbac: dict = {}
    next: dict[TileConnectionType, str] = {}  # str=id
    # subtype: ClassVar[str]
    # subtype: any | None = None



""" Request Status Codes Definition """

from pydantic import BaseModel


class RequestStatusCodes(BaseModel):
    """
    Request Status Codes DTO

    Attributes
    ----------
    allow: list[int]
        A list of allowed response codes.
    retry: list[int]
        A list of re-try-able response codes.
    reauth: list[int]
        A list of re-authorize-able response codes.
    """

    allow: list[int]
    retry: list[int]
    reauth: list[int]



from pydantic import BaseModel

from .tile import Tile


class Island(BaseModel):
    id: str
    name: str
    rbac: dict = {}
    tiles: dict[str, Tile] = {}
    dimensions: tuple[int, int] | None = None



import logging
import uuid

from ...contracts.dtos.island import Island
from ...contracts.dtos.tile import Tile
from ...contracts.types.connection import TileConnectionType
from ...contracts.types.tile import TileType

logger = logging.getLogger()


class FlatIslandFactory:
    # pylint: disable=too-many-branches
    @staticmethod
    def flat(dimensions: tuple[int, int], biome: TileType) -> Island:

        # 1. blank, named island
        local_island: Island = Island(id=str(uuid.uuid4()), name="roshar", dimensions=dimensions)

        # 2. build a blank nXm island
        max_x, max_y = dimensions
        for x in range(0, max_x):
            for y in range(0, max_y):
                local_tile_name: str = f"tile_{x}_{y}"
                local_tile: Tile = Tile(id=local_tile_name, tile_type=TileType.UNKNOWN)
                local_island.tiles[local_tile.id] = local_tile

        # 3. connect the tiles (n,m)
        for x in range(0, max_x):
            for y in range(0, max_y):
                local_tile_name: str = f"tile_{x}_{y}"

                if f"tile_{x - 1}_{y}" in local_island.tiles:
                    local_island.tiles[local_tile_name].next[TileConnectionType.LEFT] = f"tile_{x - 1}_{y}"

                if f"tile_{x + 1}_{y}" in local_island.tiles:
                    local_island.tiles[local_tile_name].next[TileConnectionType.RIGHT] = f"tile_{x + 1}_{y}"

                if f"tile_{x}_{y - 1}" in local_island.tiles:
                    local_island.tiles[local_tile_name].next[TileConnectionType.UP] = f"tile_{x}_{y - 1}"

                if f"tile_{x}_{y + 1}" in local_island.tiles:
                    local_island.tiles[local_tile_name].next[TileConnectionType.DOWN] = f"tile_{x}_{y + 1}"

        # 4. set tile types
        # outer water
        for x in range(0, max_x):
            for y in range(0, max_y):
                if x in [0, max_x - 1] or y in [0, max_y - 1]:
                    local_tile_name: str = f"tile_{x}_{y}"
                    local_island.tiles[local_tile_name].tile_type = TileType.WATER

        # 5. build shore and dirt
        for x in range(0, max_x):
            for y in range(0, max_y):
                local_tile_name: str = f"tile_{x}_{y}"
                if local_island.tiles[local_tile_name].tile_type == TileType.UNKNOWN:
                    msg: str = f"{local_tile_name}, type: {local_island.tiles[local_tile_name].tile_type}"
                    logger.debug(msg)
                    for connect_type, adjacent_id in local_island.tiles[local_tile_name].next.items():
                        # pylint: disable=line-too-long
                        msg: str = (
                            f"    direction: {connect_type}, adjacent_id: {adjacent_id}, type: {local_island.tiles[adjacent_id].tile_type}"
                        )
                        logger.debug(msg)
                        if local_island.tiles[adjacent_id].tile_type == TileType.WATER:
                            logger.debug("    water, we are shore, stopping ...")
                            local_island.tiles[local_tile_name].tile_type = TileType.SHORE
                            break
                    # see if we got assigned shore..
                    if local_island.tiles[local_tile_name].tile_type == TileType.UNKNOWN:
                        logger.debug("    still have undecided tile type .. (making biome type)")
                        local_island.tiles[local_tile_name].tile_type = biome

        return local_island



import requests

from ....contracts.dtos.requests.island_create_request import IslandCreateRequest
from ....contracts.dtos.responses.island_create_response import IslandCreateResponse
from ....contracts.dtos.responses.response import Response
from ..abstract import AbstractCommand


class IslandCreateCommand(AbstractCommand):
    def execute(self, request: IslandCreateRequest) -> IslandCreateResponse:
        response: requests.Response = requests.post(
            url=f"http://{self.options.tld}/island",
            timeout=self.options.timeout,
            json=request.model_dump(),
        )
        return Response[IslandCreateResponse].model_validate(response.json()).data



import requests

from ....contracts.dtos.responses.islands_get_response import IslandsGetResponse
from ....contracts.dtos.responses.response import Response
from ..abstract import AbstractCommand


class IslandsGetCommand(AbstractCommand):
    def execute(self) -> IslandsGetResponse:
        response: requests.Response = requests.get(
            url=f"http://{self.options.tld}/islands", timeout=self.options.timeout
        )
        return Response[IslandsGetResponse].model_validate(response.json()).data



import logging
import secrets
import uuid

from ...contracts.dtos.island import Island
from ...contracts.dtos.tile import Tile
from ...contracts.types.connection import TileConnectionType
from ...contracts.types.tile import TileType

logger = logging.getLogger()


class FlatLandFactory:
    # pylint: disable=too-many-branches
    @staticmethod
    def generate(dimensions: tuple[int, int], biome: TileType) -> Island:

        # 1. blank, named island
        local_island: Island = Island(id=str(uuid.uuid4()), name="roshar", dimensions=dimensions)

        # 2. build a blank nXm island with water
        max_x, max_y = dimensions
        for x in range(0, max_x):
            for y in range(0, max_y):
                local_tile_name: str = f"tile_{x}_{y}"
                local_tile: Tile = Tile(id=local_tile_name, tile_type=TileType.WATER)
                local_island.tiles[local_tile.id] = local_tile

        # 3. connect the tiles (n,m)
        for x in range(0, max_x):
            for y in range(0, max_y):
                local_tile_name: str = f"tile_{x}_{y}"

                if f"tile_{x - 1}_{y}" in local_island.tiles:
                    local_island.tiles[local_tile_name].next[TileConnectionType.LEFT] = f"tile_{x - 1}_{y}"

                if f"tile_{x + 1}_{y}" in local_island.tiles:
                    local_island.tiles[local_tile_name].next[TileConnectionType.RIGHT] = f"tile_{x + 1}_{y}"

                if f"tile_{x}_{y - 1}" in local_island.tiles:
                    local_island.tiles[local_tile_name].next[TileConnectionType.UP] = f"tile_{x}_{y - 1}"

                if f"tile_{x}_{y + 1}" in local_island.tiles:
                    local_island.tiles[local_tile_name].next[TileConnectionType.DOWN] = f"tile_{x}_{y + 1}"

        # 4. set tile types
        # outer water
        for x in range(0, max_x):
            for y in range(0, max_y):
                if x in [0, max_x - 1] or y in [0, max_y - 1]:
                    local_tile_name: str = f"tile_{x}_{y}"
                    local_island.tiles[local_tile_name].tile_type = TileType.WATER

        # # 5. build shore and dirt
        # for x in range(0, max_x):
        #     for y in range(0, max_y):
        #         local_tile_name: str = f"tile_{x}_{y}"
        #         if local_island.tiles[local_tile_name].tile_type == TileType.UNKNOWN:
        #             msg: str = f"{local_tile_name}, type: {local_island.tiles[local_tile_name].tile_type}"
        #             logger.debug(msg)
        #             for connect_type, adjacent_id in local_island.tiles[local_tile_name].next.items():
        #                 # pylint: disable=line-too-long
        #                 msg: str = (
        #                     f"    direction: {connect_type}, adjacent_id: {adjacent_id}, type: {local_island.tiles[adjacent_id].tile_type}"
        #                 )
        #                 logger.debug(msg)
        #                 if local_island.tiles[adjacent_id].tile_type == TileType.WATER:
        #                     logger.debug("    water, we are shore, stopping ...")
        #                     local_island.tiles[local_tile_name].tile_type = TileType.SHORE
        #                     break
        #             # see if we got assigned shore..
        #             if local_island.tiles[local_tile_name].tile_type == TileType.UNKNOWN:
        #                 logger.debug("    still have undecided tile type .. (making biome type)")
        #                 local_island.tiles[local_tile_name].tile_type = biome

        # 5. randomally seed dirt
        # for x in range(0, max_x):
        #     for y in range(0, max_y):
        #         local_tile_name: str = f"tile_{x}_{y}"

        # 5. Apply random noise
        SPAWN_RATE: int = 90  # [30/100]
        SPAWN_TYPE: TileType = biome  # TileType.DIRT
        for x in range(1, max_x - 1):
            for y in range(1, max_y - 1):
                if secrets.randbelow(100) <= SPAWN_RATE:
                    # then we spawn the tile type
                    local_tile_name: str = f"tile_{x}_{y}"
                    local_island.tiles[local_tile_name].tile_type = SPAWN_TYPE

        # 6. Create Shore
        for x in range(0, max_x):
            for y in range(0, max_y):
                local_tile_name: str = f"tile_{x}_{y}"

                # for tiles what are solid, TileType.SHORE is rechecked here... do we want to long term?
                if local_island.tiles[local_tile_name].tile_type not in [TileType.UNKNOWN, TileType.WATER]:
                    next_to_water: bool = False
                    for _, adjecent_id in local_island.tiles[local_tile_name].next.items():
                        if local_island.tiles[adjecent_id].tile_type == TileType.WATER:
                            next_to_water = True
                            break
                    if next_to_water:
                        local_island.tiles[local_tile_name].tile_type = TileType.SHORE

        return local_island



""" Wrapped Request Definition """

from pydantic import BaseModel

from ..types.request_verb import RequestVerbType
from .request_status_codes import RequestStatusCodes


class WrappedRequest(BaseModel):
    """
    Wrapped Request DTO

    Attributes
    ----------
    verb: RequestVerbType
        The verb of the REST API request.
    statuses: RequestStatusCodes
        The status code assignments for response handling.
    url: str
        The complete url to call
    data: dict | None  # str or dict - needs confirmation
        Either form data or body (based on type - see requests documentation)
    params: dict[str, str] | None
        Query parameters
    """

    verb: RequestVerbType
    statuses: RequestStatusCodes
    url: str
    data: dict | None
    params: dict[str, str] | None



import requests

from ....contracts.dtos.requests.island_delete_request import IslandDeleteRequest
from ....contracts.dtos.responses.island_delete_response import IslandDeleteResponse
from ....contracts.dtos.responses.response import Response
from ..abstract import AbstractCommand


class IslandDeleteCommand(AbstractCommand):
    def execute(self, request: IslandDeleteRequest) -> IslandDeleteResponse:
        response: requests.Response = requests.delete(
            url=f"http://{self.options.tld}/island/{request.id}", timeout=self.options.timeout
        )
        return Response[IslandDeleteResponse].model_validate(response.json()).data



import requests

from ....contracts.dtos.requests.island_get_request import IslandGetRequest
from ....contracts.dtos.responses.island_get_response import IslandGetResponse
from ....contracts.dtos.responses.response import Response
from ..abstract import AbstractCommand


class IslandGetCommand(AbstractCommand):
    def execute(self, request: IslandGetRequest) -> IslandGetResponse:
        response: requests.Response = requests.get(
            url=f"http://{self.options.tld}/island/{request.id}", timeout=self.options.timeout
        )
        return Response[IslandGetResponse].model_validate(response.json()).data



""" Command Options Definition """

from pydantic import BaseModel


class CommandOptions(BaseModel):
    """
    Command Options DTO

    Attributes
    ----------
    sleep_time: float
        Sleep time in seconds.
    retry_count: int
        Maximum number of retries.
    tld: str
        Top Level Domain to work within
    timeout: float
        Time out in seconds.
    """

    sleep_time: float
    retry_count: int
    tld: str
    timeout: float



import logging
import sys

from ..contracts.errors.service import ServiceError
from ..services.world import WorldService

logger = logging.getLogger()


class ContextManager:
    # Application Level Services
    world_service: WorldService | None = None

    def __init__(self):
        if ContextManager.world_service is None:
            ContextManager.world_service = WorldService()
            logger.debug("[ContextManager] assigned new world service to context manager")
        else:
            logger.debug("[ContextManager] world service already assigned")


try:
    ContextManager()
except ServiceError as error:
    logger.error(error)
    sys.exit(1)



""" Request Verb Definition """

from enum import Enum


class RequestVerbType(str, Enum):
    """Defines the types of requests supported."""

    GET = "get"
    POST = "post"
    PUT = "put"
    DELETE = "delete"
    PATCH = "patch"
    HEAD = "head"
    OPTIONS = "options"



import uuid

from ...contracts.dtos.island import Island
from ...contracts.dtos.world import World
from ...contracts.types.tile import TileType
from ..island.island import IslandFactory


class WorldFactory:
    @staticmethod
    def generate() -> World:
        # 1. blank, named world
        return World(
            id=str(uuid.uuid4()),
            name="darkness",
        )

    @staticmethod
    def island_discover(target_world: World, dimensions: tuple[int, int], biome: TileType) -> str:
        local_island: Island = IslandFactory.flatland(dimensions=dimensions, biome=biome)
        target_world.islands[local_island.id] = local_island
        return local_island.id



import logging

from fastapi import APIRouter

from ...contracts.dtos.responses.response import Response

logger = logging.getLogger()

router: APIRouter = APIRouter(
    prefix="/metrics",
    tags=["metrics"],
    responses={404: {"description": "Not found"}},
)


@router.get("/health")
async def health_get() -> Response[dict]:
    response: Response[dict] = Response[dict](data={"healthy": True})
    return response



# Excluding server from package exports (explicitly)

from .client.client import Client
from .client.commands.abstract import AbstractCommand
from .client.commands.island.island_create import IslandCreateCommand
from .client.commands.island.island_get import IslandGetCommand
from .client.commands.metrics.health_get import HealthGetCommand
from .common.config.environment import (
    demand_env_var,
    demand_env_var_as_bool,
    demand_env_var_as_float,
    demand_env_var_as_int,
    get_env_var,
)
from .contracts.dtos.command_options import CommandOptions
from .contracts.dtos.island import Island
from .contracts.dtos.request_status_codes import RequestStatusCodes
from .contracts.dtos.requests.island_create_request import IslandCreateRequest
from .contracts.dtos.requests.island_get_request import IslandGetRequest
from .contracts.dtos.responses.island_create_response import IslandCreateResponse
from .contracts.dtos.responses.response import Response
from .contracts.dtos.tile import Tile
from .contracts.dtos.world import World
from .contracts.dtos.wrapped_request import WrappedRequest
from .contracts.errors.environment_variable_not_found import EnvironmentVariableNotFoundError
from .contracts.errors.exceeded_retry_count import ExceededRetryCountError
from .contracts.errors.factory import FactoryError
from .contracts.errors.request_failure import RequestFailureError
from .contracts.errors.service import ServiceError
from .contracts.types.connection import TileConnectionType
from .contracts.types.request_verb import RequestVerbType
from .contracts.types.tile import TileType
from .factories.island.flat import FlatIslandFactory
from .factories.island.island import IslandFactory
from .factories.world.world import WorldFactory
from .services.world import WorldService



""" Abstract Command Definition """

import json
import logging
import time
from abc import abstractmethod

import requests
from pydantic import BaseModel
from requests import Response

from ...common.config.environment import demand_env_var, demand_env_var_as_float, demand_env_var_as_int
from ...contracts.dtos.command_options import CommandOptions
from ...contracts.dtos.wrapped_request import WrappedRequest
from ...contracts.errors.exceeded_retry_count import ExceededRetryCountError
from ...contracts.errors.request_failure import RequestFailureError
from ...contracts.types.request_verb import RequestVerbType


class AbstractCommand(BaseModel):
    options: CommandOptions | None = None

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

        if self.options is None:
            self.options: CommandOptions = CommandOptions(
                sleep_time=demand_env_var_as_float(name="DARKNESS_SERVICE_SLEEP_TIME"),
                retry_count=demand_env_var_as_int(name="DARKNESS_SERVICE_RETRY_COUNT"),
                tld=demand_env_var(name="DARKNESS_TLD"),
                timeout=demand_env_var_as_float(name="DARKNESS_SERVICE_TIMEOUT"),
            )

    @abstractmethod
    def execute(self, *args, **kwargs) -> any:
        """Command entry point"""

    def _build_requests_params(self, request: WrappedRequest) -> dict:
        """
        Builds the `requests` call parameters.

        Parameters
        ----------
        request: WrappedRequest
            The request to make.

        Returns
        -------
        params: dict
            A dictionary suitable for splatting into the `requests` call.
        """

        params: dict = {
            "url": request.url,
            "timeout": self.options.timeout,
        }
        if request.verb == RequestVerbType.POST:
            params["json"] = request.data
        if request.params is not None:
            params["params"] = request.params
        return params

    def _api_caller(self, request: WrappedRequest, depth: int) -> dict | None:
        """
        Wrapper for calls with `requests` to external APIs.

        Parameters
        ----------
        request: WrappedRequest
            Request to make.
        depth: int
            Call depth of the recursive call (retry)

        Returns
        -------
        response: dict
            A dictionary of the response.
        """

        if depth < 1:
            raise ExceededRetryCountError(json.dumps({"request": request.dict(), "depth": depth}))
        depth -= 1

        params: dict = self._build_requests_params(request=request)
        # pylint: disable=broad-except
        try:
            if request.verb == RequestVerbType.GET:
                response: Response = requests.get(**params)
            elif request.verb == RequestVerbType.POST:
                response: Response = requests.post(**params)
            elif request.verb == RequestVerbType.DELETE:
                response: Response = requests.delete(**params)
            else:
                raise Exception("Unknown Verb")
        except requests.exceptions.ConnectionError as error:
            logging.debug("Connection Error (%s) - Retrying.. %i", str(error), depth)
            time.sleep(self.options.sleep_time)
            return self._api_caller(request=request, depth=depth)
        except Exception as error:
            logging.debug("Exception needed to cover: %s", str(error))
            time.sleep(self.options.sleep_time)
            return self._api_caller(request=request, depth=depth)

        if response.status_code in request.statuses.allow:
            if response.content == b"":
                return None
            return response.json()

        if response.status_code in request.statuses.retry:
            time.sleep(self.options.sleep_time)
            return self._api_caller(request=request, depth=depth)

        if response.status_code in request.statuses.reauth:
            self._authenticate()
            return self._api_caller(request=request, depth=depth)

        raise RequestFailureError(
            json.dumps(
                {
                    "status_code": response.status_code,
                    "request": request.dict(),
                    "depth": depth,
                }
            )
        )

    def wrapped_request(self, request: WrappedRequest) -> dict | None:
        """
        High level request method.  Entry point for consumption.


        Parameters
        ----------
        request: WrappedRequest
            The request to make.

        Returns
        -------
        response: dict
            The response as a dictionary.
        """

        return self._api_caller(request=request, depth=self.options.retry_count)



from pydantic import BaseModel

from ...contracts.dtos.island import Island
from ...contracts.types.tile import TileType
from .flat import FlatIslandFactory
from .flatland import FlatLandFactory


class IslandFactory(BaseModel):
    @staticmethod
    def flat(dimensions: tuple[int, int], biome: TileType) -> Island:
        return FlatIslandFactory.flat(dimensions=dimensions, biome=biome)

    @staticmethod
    def flatland(dimensions: tuple[int, int], biome: TileType) -> Island:
        return FlatLandFactory.generate(dimensions=dimensions, biome=biome)



import logging

import uvicorn
from fastapi import FastAPI

from .routers import island, islands, metrics

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


def main():
    logger.info("[Main] starting")
    app = FastAPI()

    logger.debug("[Main] adding metrics routes")
    app.include_router(metrics.router)

    logger.debug("[Main] adding island routes")
    app.include_router(island.router)

    logger.debug("[Main] adding islands routes")
    app.include_router(islands.router)

    logger.info("[Main] online")

    uvicorn.run(app, host="0.0.0.0", port=8000, log_level=logging.getLogger().level)


if __name__ == "__main__":
    main()



import logging

from fastapi import APIRouter

from ...contracts.dtos.responses.islands_get_response import IslandsGetResponse
from ...contracts.dtos.responses.response import Response
from ..context import ContextManager

logger = logging.getLogger()

router: APIRouter = APIRouter(
    prefix="/islands",
    tags=["islands"],
)


@router.get("/")
async def island_ids_get() -> Response[IslandsGetResponse]:
    logger.debug("[GET][/islands]")
    island_ids: list[str] = ContextManager.world_service.islands_get()
    response = Response[IslandsGetResponse](data=IslandsGetResponse(ids=island_ids))
    msg: str = f"[GET][/islands] {response}"
    logger.debug(msg)
    return response



import logging

from fastapi import APIRouter, HTTPException

from ...contracts.dtos.island import Island
from ...contracts.dtos.requests.island_create_request import IslandCreateRequest
from ...contracts.dtos.responses.island_create_response import IslandCreateResponse
from ...contracts.dtos.responses.island_get_response import IslandGetResponse
from ...contracts.dtos.responses.response import Response
from ...contracts.errors.service import ServiceError
from ..context import ContextManager

logger = logging.getLogger()

router: APIRouter = APIRouter(
    prefix="/island",
    tags=["island"],
    responses={404: {"description": "Not found"}},
)


@router.post("/")
async def island_create(island_create_request: IslandCreateRequest) -> Response[IslandCreateResponse]:
    logger.debug("[POST][/island]")
    island_id: str = ContextManager.world_service.island_create(request=island_create_request)
    response = Response[IslandCreateResponse](data=IslandCreateResponse(id=island_id))
    msg: str = f"[GET][/island/create] {response}"
    logger.debug(msg)
    return response


@router.get("/{island_id}")
async def island_get(island_id: str) -> Response[IslandGetResponse]:
    msg: str = f"[GET][/island/{island_id}]"
    logger.debug(msg)

    try:
        island: Island = ContextManager.world_service.island_get(id=island_id)
    except ServiceError as error:
        logger.warning(str(error))
        raise HTTPException(status_code=404, detail=str(error)) from error

    response = Response[IslandGetResponse](data=IslandGetResponse(island=island))
    msg: str = f"[GET][/island/{response.data.island.id}] {response.model_dump()}"
    logger.debug(msg)
    return response


@router.delete("/{island_id}")
async def island_delete(island_id: str) -> None:
    msg: str = f"[DELETE][/island/{island_id}]"
    logger.debug(msg)

    try:
        ContextManager.world_service.island_delete(id=island_id)
    except ServiceError as error:
        logger.warning(str(error))
        raise HTTPException(status_code=404, detail=str(error)) from error

    msg: str = f"[DELETE][/island/{island_id}]"
    logger.debug(msg)



""" Helper functions for environment variables. """

import os

from ...contracts.errors.environment_variable_not_found import EnvironmentVariableNotFoundError


def demand_env_var(name: str) -> str:
    """
    Returns an environment variable as a string, or throws an exception.

    Parameters
    ----------
    name: str
        The name of the environment variable.

    Returns
    -------
        The environment variables value as a string.
    """

    if name not in os.environ:
        raise EnvironmentVariableNotFoundError(f"Environment variable ({name}) not found")
    return os.environ[name]


def get_env_var(name: str) -> str | None:
    """
    Returns an environment variable as a string, otherwise `None` if it does not exist.

    Parameters
    ----------
    name: str
        The name of the environment variable.

    Returns
    -------
        The environment variables value as a string, or `None` if it does not exist.
    """

    if name not in os.environ:
        return None
    return os.environ[name]


def demand_env_var_as_int(name: str) -> int:
    """
    Returns an environment variable as an int, or throws an exception.

    Parameters
    ----------
    name: str
        The name of the environment variable.

    Returns
    -------
        The environment variables value as an int.
    """

    return int(demand_env_var(name=name))


def demand_env_var_as_float(name: str) -> float:
    """
    Returns an environment variable as a float, or throws an exception.

    Parameters
    ----------
    name: str
        The name of the environment variable.

    Returns
    -------
        The environment variables value as a float.
    """

    return float(demand_env_var(name=name))


def demand_env_var_as_bool(name: str) -> bool:
    """
    Returns an environment variable as a bool, or throws an exception.

    Parameters
    ----------
    name: str
        The name of the environment variable.

    Returns
    -------
        The environment variables value as a bool.
    """

    value_str: str = demand_env_var(name=name).lower()
    if value_str in ("true", "1"):
        return True
    if value_str in ("false", "0"):
        return False
    raise EnvironmentVariableNotFoundError(f"Environment variable ({name}) not boolean and can not be loaded")



import json
import logging
from pathlib import Path

from ..contracts.dtos.island import Island
from ..contracts.dtos.requests.island_create_request import IslandCreateRequest
from ..contracts.dtos.world import World
from ..contracts.errors.service import ServiceError
from ..factories.world.world import WorldFactory

logger = logging.getLogger()

STORAGE_BASE_PATH: Path = Path(".") / "data"
METADATA_PATH: Path = STORAGE_BASE_PATH / "metadata.json"


class WorldService:
    world: World | None = None
    # islands: dict[str, Island] = {}

    # # island_ids
    # islands: dict[str, str] = {}

    def __init__(self):
        STORAGE_BASE_PATH.mkdir(parents=True, exist_ok=True)

        if not METADATA_PATH.exists():
            self._generate()
            self._commit()
        else:
            self._read()

    ### Internal ##################################

    def _generate(self):
        logger.debug("[WorldService] generating world skeleton")
        self.world = WorldFactory.generate()

    def _read(self) -> None:
        logger.debug("[WorldService] loading world data from storage")
        try:
            if self.world:
                del self.world
                self.world = None
            self.world = World.parse_file(path=METADATA_PATH)
        except json.decoder.JSONDecodeError as error:
            error_message: str = f"[WorldService] Unable to load world data from ({METADATA_PATH})"
            logger.error(error_message)
            raise ServiceError(error_message) from error

    def _commit(self):
        logger.debug("[WorldService] writing world data to storage")
        world_data: str = self.world.model_dump_json(indent=4)
        with open(file=METADATA_PATH.resolve().as_posix(), mode="w", encoding="utf-8") as file:
            file.write(world_data)

    ### Islands ##################################

    def island_create(self, request: IslandCreateRequest) -> str:
        logger.debug("[WorldService] creating island")
        # discover an island and return its id.
        island_id: str = WorldFactory.island_discover(
            target_world=self.world, dimensions=request.dimensions, biome=request.biome
        )
        self._commit()
        return island_id

    def island_delete(self, id: str) -> None:
        msg: str = f"[WorldService] deleting island {id}"
        logger.debug(msg)
        if id in self.world.islands:
            del self.world.islands[id]
        self._commit()

    def island_get(self, id: str) -> Island:
        msg: str = f"[WorldService] getting island {id}"
        logger.debug(msg)

        try:
            island: Island = self.world.islands[id]
        except KeyError as error:
            msg: str = f"Unable to find island {id}"
            logger.error(msg)
            raise ServiceError(msg) from error
        return island

    def island_put(self, island: Island) -> None:
        msg: str = f"[WorldService] putting island {island.id} into world storage"
        logger.debug(msg)
        self.world.islands[island.id] = island
        self._commit()

    def islands_get(self) -> list[str]:
        results = [key for key in self.world.islands.keys()]
        print(results)
        return results



from ..contracts.dtos.command_options import CommandOptions
from ..contracts.dtos.island import Island
from ..contracts.dtos.requests.island_create_request import IslandCreateRequest
from ..contracts.dtos.requests.island_get_request import IslandGetRequest
from ..contracts.dtos.responses.island_create_response import IslandCreateResponse
from ..contracts.dtos.responses.island_get_response import IslandGetResponse
from ..contracts.dtos.responses.islands_get_response import IslandsGetResponse
from ..contracts.types.tile import TileType
from .commands.island.island_create import IslandCreateCommand
from .commands.island.island_get import IslandGetCommand
from .commands.islands.islands_get import IslandsGetCommand
from .commands.metrics.health_get import HealthGetCommand


class Client:
    health_get_command: HealthGetCommand

    island_create_command: IslandCreateCommand
    island_get_command: IslandGetCommand
    islands_get_command: IslandsGetCommand

    def __init__(self, options: CommandOptions | None = None):
        command_dict: dict = {}
        if options:
            command_dict["options"] = options

        ### Metrics
        self.health_get_command = HealthGetCommand.model_validate(command_dict)

        ### /island
        self.island_create_command = IslandCreateCommand.model_validate(command_dict)
        self.island_get_command = IslandGetCommand.model_validate(command_dict)

        ### /islands
        self.islands_get_command = IslandsGetCommand.model_validate(command_dict)

    def health_get(self) -> dict:
        """
        Gets the server health.

        Returns
        -------
        health: dict
            The server health (true or false).
        """

        return self.health_get_command.execute()

    def island_create(self, dimensions: tuple[int, int], biome: TileType) -> str:
        request: IslandCreateRequest = IslandCreateRequest(dimensions=dimensions, biome=biome)
        response: IslandCreateResponse = self.island_create_command.execute(request=request)
        return response.id

    def island_get(self, id: str) -> Island:
        request: IslandGetRequest = IslandGetRequest(id=id)
        response: IslandGetResponse = self.island_get_command.execute(request=request)
        return response.island

    def islands_get(self) -> list[str]:
        response: IslandsGetResponse = self.islands_get_command.execute()
        return response.ids



""" Health Get Command Definition """

import requests

from ....contracts.dtos.responses.response import Response
from ..abstract import AbstractCommand


class HealthGetCommand(AbstractCommand):
    """
    Health Get Command
    Gets the server health.

    Methods
    -------
    execute(self) -> dict
        Executes the command.
    """

    def execute(self) -> dict:
        """
        Executes the command.

        Returns
        -------
        health: dict
            The server health (true or false).
        """

        response: requests.Response = requests.get(
            url=f"http://{self.options.tld}/metrics/health",
            timeout=self.options.timeout,
        )
        return Response[dict].model_validate(response.json()).data
